<?php
Namespace App\Controller\Admin\Inventory;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearBuilding;
use App\Model\Admin\HomegearStory;

class BuildingController extends Controller
{
    public function indexAction(Request $request, HomegearBuilding $building)
    {
        return $this->render('Admin/Inventory/Building/list.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'buildings' => $building->loadAll(true)
        ));
    }

    public function createAction(Request $request, HomegearBuilding $building)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'name' => new Assert\NotBlank,
            'lang' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $building->create($params['name'],$params['lang']);
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.buildings');
    }

    public function editAction($id, Request $request, HomegearBuilding $building, HomegearStory $story)
    {
        // get the current building
        $currentBuilding = $building->loadById($id,true);
        // get all known stories
        $stories = $story->loadAll();
        if (array_key_exists('STORIES',$currentBuilding) && $currentBuilding['STORIES'] !== NULL) {
            foreach($currentBuilding['STORIES'] as $k => $v) {
                // find device-data for stories in current building
                $key = array_search($v['id'], array_column($stories, 'ID'));
                if (isset($key) && $key !== NULL) {
                    $sort = $currentBuilding['STORIES'][$k]['sort'];
                    $currentBuilding['STORIES'][$k] = ['id' => $stories[$key]['ID'], 'name' => $stories[$key]['TRANSLATIONS'], 'sort' => $sort];
                    $stories[$key]['inBuilding'] = true;
                }
            }
        }
        return $this->render('Admin/Inventory/Building/edit.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'building' => $currentBuilding,
            'allStories' => $stories
        ));
    }

    public function updateAction($id, Request $request, HomegearBuilding $building)
    {
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'name' => new Assert\NotBlank,
            'goback' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $building->update($id,$params['name']);
        }
        if (array_key_exists('goback',$params) && $params['goback'] != '0') {
            return $this->redirectToRoute($params['goback']);
        } else {
            return $this->redirectToRoute('admin.inventory.buildings.edit', array('id' => $id));
        }
    }

    public function deleteAction($id, Request $request, HomegearBuilding $building)
    {
        $building->delete($id);
        return $this->redirectToRoute('admin.inventory.buildings');
    }

    public function updateSortAction($id, Request $request, HomegearBuilding $building)
    {
        $params = $request->request->all();
        $building->updateSort($id,$params['position']);
        return new JsonResponse(array('success' => 1));
    }

    public function addStoryAction($id, Request $request, HomegearBuilding $building)
    {
        $params = $request->request->all();
        $building->addStory($id,$params['story']);
        return new JsonResponse(array('success' => 1));
    }

    public function removeStoryAction($id, Request $request, HomegearBuilding $building)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'storyid' => new Assert\NotBlank
        ));
        $params = $request->query->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $building->removeStory($id,$params['storyid']);
        }
        return $this->redirectToRoute('admin.inventory.buildings.edit', array('id' => $id));
    }

    public function updateStorySortAction($id, Request $request, HomegearBuilding $building)
    {
        $params = $request->request->all();
        $building->updateStorySort($id,$params['storyId'],$params['position']);
        return new JsonResponse(array('success' => 1));
    }

}