<?php
Namespace App\Controller\Admin\Inventory;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearRoom;
use App\Model\Admin\HomegearStory;
use App\Model\Admin\HomegearDevice;

class RoomController extends Controller
{
    public function indexAction(Request $request, HomegearRoom $room, HomegearStory $story)
    {
        $stories = $story->loadAll(true);
        $rooms = $room->loadAll($stories);
        return $this->render('Admin/Inventory/Room/list.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'rooms' => $rooms,
            'stories' => $stories
        ));
    }

    public function editAction($id, Request $request, HomegearRoom $room, HomegearDevice $device)
    {
        // get the current room
        $currentRoom = $room->loadById($id);
        // get all known devices
        $devices = $device->loadAll();
        if (array_key_exists('devices',$currentRoom) && $currentRoom['devices'] !== NULL) {
            foreach($currentRoom['devices'] as $k => $v) {
                // find device-data for devices in current room
                if ($v['ID'] == 'systemVariable') { continue; }
                $key = array_search($v['ID'], array_column($devices, 'ID'));
                if (isset($key) && $key !== NULL) {
                    $currentRoom['devices'][$k]['name'] = $devices[$key]['name'];
                    $devices[$key]['inRoom'] = true;
                }
            }
        }
        return $this->render('Admin/Inventory/Room/edit.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'room' => $currentRoom,
            'allDevices' => $devices
        ));
    }

    public function updateAction($id, Request $request, HomegearRoom $room)
    {
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'name' => new Assert\NotBlank,
                'goback' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $room->update($id,$params['name']);
            if (array_key_exists('roomIcon',$params)) {
                $metadata = $room->getMetadata();
                $metadata['icon'] = $params['roomIcon'];
                $room->setMetadata($id, $metadata);
            }
        }
        // TODO: check result and redirect with flash
        if (array_key_exists('goback',$params) && $params['goback'] != '0') {
            return $this->redirectToRoute($params['goback']);
        } else {
            return $this->redirectToRoute('admin.inventory.rooms.edit', array('id' => $id));
        }
    }

    public function createAction(Request $request, HomegearRoom $room, HomegearStory $story)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'name' => new Assert\NotBlank,
                'lang' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $room->create($params['name'],$params['lang']);
            if (array_key_exists('story',$params) && $params['story'] != '') {
                $story->addRoom($params['story'],[$result]);
            }
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.rooms');
    }

    public function deleteAction($id, Request $request, HomegearRoom $room)
    {
        $room->delete($id);
        return $this->redirectToRoute('admin.inventory.rooms');
    }

    public function updateSortAction($id, Request $request, HomegearRoom $room)
    {
        $params = $request->request->all();
        $room->updateSort($id,$params['position']);
        return new JsonResponse(array('success' => 1));
    }

    public function addDeviceAction($id, Request $request, HomegearRoom $room)
    {
        $params = $request->request->all();
        $room->addDevice($id,$params['device']);
        return new JsonResponse(array('success' => 1));
    }

    public function removeDeviceAction($id, Request $request, HomegearRoom $room)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'deviceid' => new Assert\NotBlank,
            'channel' => new Assert\NotNull,
            'variable' => new Assert\NotNull,
        ));
        $params = $request->query->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $room->removeDevice($id,$params['deviceid'],$params['channel'],$params['variable']);
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.rooms.edit', array('id' => $id));
    }

    public function updateDeviceSortAction($id, Request $request, HomegearRoom $room)
    {
        $params = $request->request->all();
        $room->updateDeviceSort($id,$params['deviceId'],$params['position'],$params['channel'],$params['variable']);
        return new JsonResponse(array('success' => 1));
    }
}