<?php
Namespace App\Controller\Admin\Settings;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearUser;
use App\Model\Admin\HomegearRoom;
use App\Model\Admin\HomegearStory;
use App\Utils\Utils;

class UserController extends Controller
{
    public function indexAction(Request $request, HomegearUser $user)
    {
        $locale = $request->getSession()->get('_locale');
        return $this->render('Admin/Settings/User/list.html.twig',array(
            'sect' => $request->attributes->get('sect'),
            'users' => $user->loadAll(),
            'groups' => $user->loadAllGroups(),
            'locale' => ($locale ? $locale : 'en-US')
        ));
    }

    public function editAction($username, Request $request, HomegearUser $user, HomegearRoom $room, HomegearStory $story, Utils $utils)
    {
        $loggedInUser = $this->getUser();
        if ($request->getMethod() == "GET") {
            $stories = $story->loadAll(true);
            $rooms = $room->loadAll($stories);
            // TODO: refractor (check for current logged in user not possible within model)
            $editUser = $user->loadByUsername($username,$loggedInUser->getUsername());
            $allGroups = $user->loadAllGroups();
            return $this->render('Admin/Settings/User/edit.html.twig',array(
                'sect' => $request->attributes->get('sect'),
                'user' => $editUser,
                'groups' => $allGroups,
                'rooms' => $rooms,
                'stories' => $stories,
                'mgmt' => $utils->checkManagementRunning(false)
            ));
        } elseif ($request->getMethod() == "POST") {
            // Validate the input
            $validator = Validation::createValidator();
            $constraint = new Assert\Collection(array(
                'fields' => array(
                    'name' => new Assert\NotBlank,
                    'locale' => new Assert\NotBlank,
                    'theme' => new Assert\NotBlank,
                    'password' => new Assert\NotNull,
                    'expert' => new Assert\NotNull
                ),
                'allowExtraFields' => true
            ));
            $params = $request->request->all();
            $errorList = $validator->validate($params, $constraint);
            if (0 === count($errorList)) {
                if (!array_key_exists('groups', $params)) {
                    $params['groups'] = array();
                }
                if (!array_key_exists('afterLogin', $params)) {
                    $params['afterLogin'] = '';
                }
                if (!array_key_exists('room', $params)) {
                    $params['room'] = '';
                }
                if (!array_key_exists('story', $params)) {
                    $params['story'] = '';
                }
                $result = $user->updateUser($params['name'],$params['password'],$params['groups'],$params['locale'],$params['afterLogin'],$params['theme'],$params['expert'],$params['room'],$params['story']);
                // check if user is updating himself, set locale if so ...
                if ($loggedInUser->getUsername() == $params['name']) {
                    $request->getSession()->set('_locale', $params['locale']);
                }
            }
            // TODO: check validation/result and redirect with flash messages
            return $this->redirectToRoute('admin.settings.users');
        }
    }

    public function createAction(Request $request, HomegearUser $user)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'username' => new Assert\NotBlank,
                'password' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            if (!array_key_exists('groups', $params)) {
                $params['groups'] = array();
            }
            $result = $user->create($params['username'],$params['password'],$params['groups'],$params['locale'],$params['theme']);
            if ($result === true) {
                // automatically generate certificate for the new user
                $cert = $user->generateCertificate($params['username']);
            }
        }
        // TODO: check validation/result and redirect with flash messages
        return $this->redirectToRoute('admin.settings.users');
    }

    public function deleteAction($username, HomegearUser $user)
    {
        // Validate the input
        // TODO: we only want to check a single string here, no array validation needed
        // see: http://symfony.com/doc/current/validation/raw_values.html
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'username' => new Assert\NotBlank,
        ));
        $errorList = $validator->validate(['username' => $username], $constraint);
        if (0 === count($errorList)) {
            $result = $user->delete($username);
        }
        // TODO: check validation/result and redirect with flash messages
        return $this->redirectToRoute('admin.settings.users');
    }

    public function generateCertificateAction($username, HomegearUser $user)
    {
        $result = $user->generateCertificate($username);
        return new JsonResponse($result);
    }
}