<?php
namespace App\Utils;

use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Translation\TranslatorInterface;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\Console\Input\ArrayInput;
use Symfony\Component\Console\Output\BufferedOutput;
use Symfony\Bundle\FrameworkBundle\Console\Application;
use \Homegear\Homegear as Homegear;

Class Utils
{
    protected $hg;
    protected $translator;
    protected $request;
    protected $router;

    public function __construct(RequestStack $requestStack, TranslatorInterface $translator, RouterInterface $router)
    {
        $this->hg = new Homegear;
        $this->translator = $translator;
        $this->request = $requestStack;
        $this->router = $router;
    }

    public function checkManagementRunning($fullstatus = true)
    {
        if (call_user_func("Homegear\\Homegear::system.listMethods", "managementGetCommandStatus")) {
            return true;
        }
        if (!$fullstatus) { return false; }
        $current = $this->request->getCurrentRequest()->get('_route');
        return $result = ['status' => '500', 'message' => $this->translator->trans('management.not_running',array(),'admin'), 'callback' => "setTimeout(checkMgmt, 5000, '".$this->router->generate('check', ['type' => 'mgmt'])."', '".$this->router->generate($current)."');"];
    }

    public function checkSmarthaManagementRunning($fullstatus = true)
    {
        if (call_user_func("Homegear\\Homegear::system.listMethods", "smarthaAptInstall")) {
            return true;
        }
        if (!$fullstatus) { return false; }
        $current = $this->request->getCurrentRequest()->get('_route');
        return $result = ['status' => '500', 'message' => $this->translator->trans('management.not_running',array(),'admin'), 'callback' => "setTimeout(checkMgmt, 5000, '".$this->router->generate('check', ['type' => 'mgmt'])."', '".$this->router->generate($current)."');"];
    }

    public function checkWebSshRunning($fullstatus = true)
    {
        if (call_user_func("Homegear\\Homegear::system.listMethods", "websshInput")) {
            return true;
        }
        if (!$fullstatus) { return false; }
        $current = $this->request->getCurrentRequest()->get('_route');
        return $result = ['status' => '500', 'message' => $this->translator->trans('webssh.not_running',array(),'admin'), 'callback' => "setTimeout(checkWebSsh, 5000, '".$this->router->generate('check', ['type' => 'webssh'])."', '".$this->router->generate($current)."');"];
    }

    public function checkCa()
    {
        if ($this->checkManagementRunning(false)) {
            return $this->hg->managementCaExists();
        }
        return -1;
    }

    public function checkReadonly()
    {
        $result = shell_exec('grep "\sro[\s,]" /proc/mounts |grep "\s/\s"');
        if (is_null($result)) { return false; }
        return true;
    }

    public function getCommandStatus($id) {
        if ($this->checkManagementRunning(false)) {
            $result = $this->hg->managementGetCommandStatus($id);
            while (!$result['finished']) {
                usleep(100000); // 100ms
                $result = $this->hg->managementGetCommandStatus($id);
            }
            return $result;
        }
        return 0;
    }

    public function waitAptFinished($fromSmartha = false)
    {
        $result = false;
        $pids = array();
        usleep(200000); // sleep a little bit (200ms), needed on slow systems
        if ($fromSmartha === true) {
            if ($this->checkSmarthaManagementRunning(false)) {
                $result = $this->hg->smarthaAptRunning();
                exec("pgrep apt-get", $pids);
                while ($result && !empty($pids)) {
                    usleep(100000); // 100ms
                    $result = $this->hg->smarthaAptRunning();
                    exec("pgrep apt-get", $pids);
                }
                return true;
            }
        } else {
            if ($this->checkManagementRunning(false)) {
                $result = $this->hg->managementAptRunning();
                exec("pgrep apt-get", $pids);
                while ($result && !empty($pids)) {
                    usleep(100000); // 100ms
                    $result = $this->hg->managementAptRunning();
                    exec("pgrep apt-get", $pids);
                }
                return true;
            }
        }
        return -1;
    }

    public function getDataPath()
    {
        $conf = file_get_contents('/etc/homegear/main.conf');
        preg_match('/^writeableDataPath = (.+)/m',$conf,$result);
        if (!is_array($result) || count($result) < 1) {
            preg_match('/^dataPath = (.+)/m',$conf,$result);
        }
        return $result[1];
    }

    public function getDatabasePath()
    {
        $conf = file_get_contents('/etc/homegear/main.conf');
        preg_match('/^databasePath = (.+)/m',$conf,$result);
        if (!is_array($result) || count($result) < 1) {
            return false;
        }
        return $result[1];
    }

    public function getDatabaseBackupPath()
    {
        $conf = file_get_contents('/etc/homegear/main.conf');
        preg_match('/^databaseBackupPath = (.+)/m',$conf,$result);
        if (!is_array($result) || count($result) < 1) {
            return false;
        }
        return $result[1];
    }

    public function getNodeBlueEnabled()
    {
        $conf = file_get_contents('/etc/homegear/main.conf');
        preg_match('/^enableNodeBlue = (.+)/m',$conf,$result);
        if (!is_array($result) || count($result) < 1) {
            return false;
        }
        if ($result[1] == "false"){
            return false;
        }
        return true;
    }

    public function saveToSession($data = false)
    {
        if ($data === false || !is_array($data)) { return; }
        $session = new Session();
        foreach ($data as $key => $value) {
            $session->set("$key", "$value");
        }
    }

    public function checkUpdateRunning($type = false)
    {
        if ($type === false) { return false; }
        $session = new Session();
        $update = $this->hg->getData('Updates','UpdateRunning');
        if ($update !== null) {
            if ($this->checkManagementRunning(false)) {
                try {
                    $result = $this->hg->managementAptRunning();
                    exec("pgrep apt-get", $pids);
                } catch(\Homegear\HomegearException $e) {
                    // return -1;
                }
                if ((isset($result) && $result === true) || (isset($pids) && !empty($pids))) {
                    if ($type != '_ui-update-running') {
                        $session->getFlashBag()->set('warning',$this->translator->trans('flash.update_running',array(),'admin'));
                    }
                    return true;
                }
                $this->hg->deleteData('Updates','UpdateRunning');
                if ($type == '_os-update-running') {
                    $this->hg->deleteData('Updates','UpdateAvailable_System');
                    $session->getFlashBag()->set('warning',$this->translator->trans('flash.update_finished',array(),'admin'));
                }
                if ($type == '_sys-update-running') {
                    $this->hg->deleteData('Updates','UpdateAvailable_Homegear');
                }
                if ($type == '_ui-update-running') {
                    $this->hg->deleteData('Updates','UpdateAvailable_UI');
                }
                return false;
            }
            return -1;
        }
        return -1;
    }

    public function getIpAddresses($int = false) {
        $interfaces = array();
        $tmp = shell_exec('ifconfig');
        $tmp = preg_split("/\n\n/",$tmp);
        $gateway = trim(shell_exec("/sbin/ip route | awk '/default/ { print $3 }'"));
        foreach ($tmp as $k => $v) {
            if ($v == '') { continue; }
            // TODO: this is not reliable!
            $interface = trim(substr($v, 0, strpos($v, ': flags')));
            if ($interface == '') {
                $interface = trim(substr($v, 0, strpos($v, 'Link')));
            }
            if (strpos($interface,'lo') === false) {
                // get ipv4 address/netmask
                preg_match('/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/', $v, $ip, PREG_OFFSET_CAPTURE);
                if (array_key_exists(0,$ip)) {
                    preg_match('/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/', $v, $netmask, PREG_OFFSET_CAPTURE, ($ip[0][1] + strlen($ip[0][0])));
                }
                // get ipv6 address/netmask
                $v6Data = explode('/',trim(shell_exec("/sbin/ip -6 -o addr show dev ".$interface." |awk '{print $4}'")));
                $v6Gateway = trim(shell_exec("/sbin/ip -6 route | grep ^default |awk '{print $3}'"));
                if (array_key_exists(0,$ip)) {
                    $octets = explode('.',$ip[0][0]);
                    $interfaces[$k]['name'] = $interface;
                    $interfaces[$k]['ip'] = $ip[0][0];
                    $interfaces[$k]['net'] = $octets[0].'.'.$octets[1].'.'.$octets[2];
                    $interfaces[$k]['lastOctet'] = $octets[3];
                    $interfaces[$k]['netmask'] = $netmask[0][0];
                    $interfaces[$k]['gateway'] = $gateway;
                    $interfaces[$k]['ipv6'] = ( (is_array($v6Data) && array_key_exists(0,$v6Data)) ? $v6Data[0] : '');
                    $interfaces[$k]['ipv6_netmask'] = ((is_array($v6Data) && array_key_exists(1,$v6Data)) ? $v6Data[1] : '');
                    $interfaces[$k]['ipv6_gateway'] = $v6Gateway;
                }
                if (($int !== false) && ($int == $interface) && array_key_exists(0,$ip)) {
                    return $interfaces[$k];
                }
            }
        }
        return array_values($interfaces);
    }

    public static function staticGetIpAddresses($int = false) {
        $interfacrs = array();
        $tmp = shell_exec('ifconfig');
        $tmp = preg_split("/\n\n/",$tmp);
        $gateway = trim(shell_exec("/sbin/ip route | awk '/default/ { print $3 }'"));
        foreach ($tmp as $k => $v) {
            if ($v == '') { continue; }
            // TODO: this is not reliable!
            $interface = trim(substr($v, 0, strpos($v, ': flags')));
            if ($interface == '') {
                $interface = trim(substr($v, 0, strpos($v, 'Link')));
            }
            if (strpos($interface,'lo') === false) {
                // get ipv4 address/netmask
                preg_match('/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/', $v, $ip, PREG_OFFSET_CAPTURE);
                if (array_key_exists(0,$ip)) {
                    preg_match('/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/', $v, $netmask, PREG_OFFSET_CAPTURE, ($ip[0][1] + strlen($ip[0][0])));
                }
                // get ipv6 address/netmask
                $v6Data = explode('/',trim(shell_exec("/sbin/ip -6 -o addr show dev ".$interface." |awk '{print $4}'")));
                $v6Gateway = trim(shell_exec("/sbin/ip -6 route | grep ^default |awk '{print $3}'"));
                if (array_key_exists(0,$ip)) {
                    $octets = explode('.',$ip[0][0]);
                    $interfaces[$k]['name'] = $interface;
                    $interfaces[$k]['ip'] = $ip[0][0];
                    $interfaces[$k]['net'] = $octets[0].'.'.$octets[1].'.'.$octets[2];
                    $interfaces[$k]['lastOctet'] = $octets[3];
                    $interfaces[$k]['netmask'] = $netmask[0][0];
                    $interfaces[$k]['gateway'] = $gateway;
                    $interfaces[$k]['ipv6'] = ( (is_array($v6Data) && array_key_exists(0,$v6Data)) ? $v6Data[0] : '');
                    $interfaces[$k]['ipv6_netmask'] = ((is_array($v6Data) && array_key_exists(1,$v6Data)) ? $v6Data[1] : '');
                    $interfaces[$k]['ipv6_gateway'] = $v6Gateway;
                }
                if (($int !== false) && ($int == $interface) && array_key_exists(0,$ip)) {
                    return $interfaces[$k];
                }
            }
        }
        return array_values($interfaces);
    }

    public function getHost()
    {
        $host = $_SERVER['HTTP_HOST'];
        if ($_SERVER['SERVER_ADDR'] != $host) {
            if (stristr($host,':')) {
                list($host,$port) = explode(':',$host);
            }
            return $host;
        }
        return $_SERVER['HTTP_HOST'];
    }

    public function getPort()
    {
        $host = $_SERVER['HTTP_HOST'];
        if (array_key_exists('HTTPS',$_SERVER) && $_SERVER['HTTPS'] == 'on') {
            $port = '443';
        } else {
            $port = '80';
        }
        if (ip2long($host) === false) {
            if (stristr($host,':')) {
                list($host,$port) = explode(':',$host);
            }
            if ((stristr($host,'cloudmatic-vpn') || stristr($host,'meine-homematic')) && $port == '80' || $port == '') {
                $port = 443;
            }
        }
        return $port;
    }

    public function checkExpert($username = null)
    {
        if ($username == null) { return false; }
        $userSettings = $this->hg->getUserMetadata($username);
        if (array_key_exists('expert', $userSettings) && $userSettings['expert'] == "true") {
            return true;
        }
        return false;
    }

    public function getTheme($username = null)
    {
        if ($username == null) { return false; }
        $userSettings = $this->hg->getUserMetadata($username);
        if (array_key_exists('theme', $userSettings)) {
            return $userSettings['theme'];
        }
        return 'light';
    }

    public function checkOss()
    {
        if (substr($_SERVER['REQUEST_URI'],0,6) !== '/admin') {
            return false;
        }
        return true;
    }

    public function checkRelease()
    {
        $uiversion = UIVERSION;
        if ($uiversion == "%%UIVERSION%%" && file_exists('../.version')) {
            $uiversion = file_get_contents('../.version');
        }
        if (stristr($uiversion,'-dev') !== FALSE) {
            return 'dev';
        }
        if (stristr($uiversion,'-beta') !== FALSE) {
            return 'beta';
        }
        return 'final';
    }

    public function checkHGS()
    {
        return defined('HGS');
    }

    public function rand_passwd( $length = 10, $chars = 'abcdefghijkmnopqrstuvwxyzABCDEFGHJKMNPQRSTUVWXYZ23456789' ) {
        return substr( str_shuffle( $chars ), 0, $length );
    }

    public function setOled($msg = null)
    {
        if (file_exists('/usr/share/smartha-oled/smartha_display.py')) {
            if (file_exists('/data/disable_oled')) {
                rename('/data/disable_oled', '/data/disable_oled_tmp');
            }
            if (file_exists('/data/disable_oled_tmp')) {
                rename('/data/disable_oled_tmp', '/data/disable_oled');
            }
            if ($msg == null) {
                exec('python /usr/share/smartha-oled/smartha_display.py');
            } else {
                exec('python /usr/share/smartha-oled/smartha_display.py -t "'.$msg.'"');
            }
        }
    }

    public function clearCache($kernel = false, $env = false)
    {
        // clear opcace
        opcache_reset();
        if ($kernel === false || $env === false) { return; }
        // clear app-cache
        $application = new Application($kernel);
        $application->setAutoExit(false); //exit after run
        $input = new ArrayInput([
            'command' => 'cache:clear',
            '--env'   => $env,
            '--no-warmup' => true
        ]);
        $output = new BufferedOutput();
        $runCode = $application->run($input, $output);
        //$content = $output->fetch();
        return true;
    }

    public function getLoglevel()
    {
        return $this->hg->logLevel();
    }

    public function setLoglevel($level = false)
    {
        if ($level === false) { return; }
        $this->hg->logLevel(intval($level));
        return $this->hg->logLevel();
    }

    public function checkDeviceExists($id = false)
    {
        if ($id === false) { return false; }
        try {
            $this->hg->getName(intval($id));
        } catch(\Homegear\HomegearException $e) {
            return false;
        }
        return true;
    }

    public function checkFamilyExists($id = false)
    {
        if ($id === false) { return false; }
        try {
            $this->hg->getPairingInfo(intval($id));
        } catch(\Homegear\HomegearException $e) {
            return false;
        }
        return true;
    }

    public function checkUserExists($username = false) {
        if ($username === false) { return false; }
        exec("cat /etc/passwd |grep $username", $user);
        if (is_array($user)) {
            foreach ($user as $key => $value) {
                $value = explode(':',$value);
                if ($value[0] == $username) { return true; }
            }
        }
        return false;
    }
}